require "#{RAILS_ROOT}/lib/progress_bar"
baseline_verbose = ENV["BASELINES_VERBOSE"].present?

task :environment

namespace :puppet do
  namespace :plugin do
    namespace :baseline do
      desc 'Prune old baselines from the databases, will print help if run without arguments'
      task :prune => [:environment] do
        units = {
          'min' => '60',
          'hr' => '3600',
          'day' => '86400',
          'wk' => '604800',
          'mon' => '2592000',
          'yr' => '31536000'
        }
        known_units = units.keys.join(',')

        usage = %{
EXAMPLE:
  # Prune records upto 1 month old:
  rake puppet:plugin:baseline:prune upto=1 unit=mon

UNITS:
  Valid units of time are: #{known_units}
    }.strip

        unless ENV['upto'] || ENV['unit']
          puts usage
          exit 0
        end

        errors = []

        if ENV['upto'] =~ /^\d+$/
          upto = ENV['upto'].to_i
        else
          errors << "You must specify how far up you want to prune as an integer, e.g.: upto={some integer}" \
        end

        if unit = ENV['unit']
          unless units.has_key?(unit)
            errors << "I don't know that unit. Valid units are: #{known_units}" \
          end
        else
          errors << "You must specify the unit of time, .e.g.: unit={#{known_units}}" \
        end

        if errors.present?
          puts errors.map { |error| "ERROR: #{error}" }
          puts
          puts usage
          exit 1
        end

        cutoff = Time.now.gmtime - (upto * units[unit].to_i)
        puts "Deleting baselines before #{cutoff}..."

        deleted_count = 0
        total_count = NodeBaseline.where(["date < ?", cutoff]).count

        pbar = ProgressBar.new("Deleting:", total_count, STDOUT)

        NodeBaseline.delete_older_than(cutoff) do
          deleted_count += 1
          pbar.inc
        end

        pbar.finish

        puts "Deleted #{deleted_count} baselines."
      end

      desc "Create missing baseline for all nodes.
This task creates all missing baselines, up to the end of the current
'day', as defined by the end-of-day configuration."
      task :create_baselines => [:environment] do
        nodes = Node.all(:conditions => 'last_inspect_report_id IS NOT NULL')
        nodes.each do |node|
          BaselineUtil.create_missing_baselines_for_node node
        end

        puts "#{nodes.count} nodes updated for baselines" if baseline_verbose
      end

      desc 'Daily node/baseline diff summary generation'
      task :diff_nodes => [:environment] do
        total = NodeBaseline.self_comparisons.count(:conditions => {:diffed => false})
        count = 0
        Summarizer.queue_diff_summarization do
          count += 1
        end
        puts "#{count} of #{total} node baseline summaries queued" if baseline_verbose
      end

      desc 'All standard daily maintenance tasks for baselines'
      task :daily => [:environment] do
        Rake::Task['puppet:plugin:baseline:create_baselines'].invoke
        Rake::Task['puppet:plugin:baseline:diff_nodes'].invoke
      end
    end
  end
end
